using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.Serialization;
using System.Data.OleDb;
using Guardant; // Используем классы Guardant

namespace Clients
{
    public partial class Form1 : Form
    {
        private config Config;
        private string connectionString;
        
        /* Начало. Блок переменных для работы с Guardant API */ 
        GrdE RetCode;                       // Код ошибки для всех функций Guardant API
        Handle GrdHandle = new Handle();    // Создает пустой хэндл для контейнера Guardant

        static uint CryptPU = 0x8566783U;         // Константы для шифрования кодов
        static uint CryptRD = 0x17d49c84U;        // доступа к ключу защиты
        static long InitVector = 327235620;       // Вектор инициализации для алгоритма GSII64
        static long InitVectorAES1 = 697394536;       // Вектор инициализации для алгоритма AES
        static long InitVectorAES2 = 478236236;       // Вектор инициализации для алгоритма AES

        // Шифруем демонстрационные коды доступа, вычитая из них случайные числа для 
        // обеспечения более высокой степени взломоучтойчивости
        // This way they are not shown in the application
        uint PublicCode = (uint)GrdDC.DEMONVK - CryptPU;    // Должно быть зашифровано        
        uint ReadCode = (uint)GrdDC.DEMORDO - CryptRD;    // Должно быть зашифровано
        
        // Переменные используются в функции GrdSetFindMode()
        GrdFMR RemoteMode;         // Флаги поиска локальных или сетевых ключей                  
        GrdFM DongleFlags;        // Флаги поиска ключа                  
        uint ProgramNumber = 0;  // Номер программы                      
        uint Version = 0;        // Версия программы                                 
        uint DongleID = 0;       // ID ключа (Номер ключа)
//        uint ModelID = 0;        // Модель ключа
        byte[] TypeID = new byte[2];// Тип ключа
        uint SerialNumber;       // Сериный номер                           
        uint BitMask;            // Битовая маска                           
        GrdDT DongleType;         // Тип ключа                            
        GrdFMM DongleModel;        // Модель ключа                           
        GrdFMI DongleInterface;    // Тип интерфейса ключа

        byte[] InitVectorAES = new byte[16];       // Переменная вектора инициализации для алгоритма AES
        string SecretKeyAESStr;                    // Специальный ключ шифрования строка  
        byte[] SecretKeyAES = new byte[32];        // Специальный ключ шифрования массив  
        byte[] GrdAES_Context = new byte[(int)GrdAES.CONTEXT_SIZE]; // Буфер для программно-реализованного алгоритма шифрования

        /* Конец. Блок переменных для работы с Guardant API */

        int ClientID; // Хранится ID клиента из БД
        string ClientName; // Хранится имя клиента из БД
        UInt32 ClientIDIntV; // Будет содержать вектор инициализации кодирования 
        string ClientComments; // Строка комментарий
        byte[] ClientCommentsB; // Массив комментарий
        string ClientIDStr; // Строка с номером клиента


        public Form1()
        {
            InitializeComponent();

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            /* ----------- Начало блока работы с Guardant API ------------------ */

            // Инициализация копии GrdAPI. GrdStartup() должен быть вызван перед первым вызовом функций GrdAPI 
            RetCode = GrdApi.GrdStartup(GrdFMR.Local);	// Мы будем использовать только локальные ключи
            ErrorHandling(GrdHandle, RetCode);

            // -----------------------------------------------------------------
            // Создание контейнера Guardant и получение его хэндла
            // -----------------------------------------------------------------
            GrdHandle = GrdApi.GrdCreateHandle(GrdCHM.MultiThread);
            if (GrdHandle.Address == 0) // Если найдена ошибка
            {
                ErrorHandling(new Handle(0), GrdE.MemoryAllocation);
            }

            // -----------------------------------------------------------------
            // Установка кодов доступа к ключу защиты Guardant
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdSetAccessCodes(GrdHandle,	// хэндл контейнера Guardant 
                PublicCode + CryptPU,   // Публичный код. Используется для поиска ключа. Должен быть установлен
                ReadCode + CryptRD,     // Приватный код для чтения; Данный код используется для чтения незащищенных ячеек памяти;
                0,	// Приватный код для записи. В большинстве случаев, он не нужен. Старайтесь не включать его в итоговые версии приложения.
                0);  // Мастер код. Используется для редактирования параметров ключа. Никогда не вставляйте этот код в публичные версии приложения!

            ErrorHandling(GrdHandle, RetCode); // Обработка ошибок


            // -----------------------------------------------------------------
            // Устанавливаем параметры поиска ключа
            // -----------------------------------------------------------------
            RemoteMode = GrdFMR.Local; 				            // Поиск только локальных ключей
            DongleFlags = GrdFM.NProg & GrdFM.Mask;	            // Устанавливаем флаг поиска по номеру программы и маске
            ProgramNumber = 5;     								// Устанавливаем номер программы. В нашем поиске не используется.                
            DongleID = 0;       		     					// ID ключа. Не используется
            SerialNumber = 0;     							    // Серийный номер. В нашем поиске не используется.                     
            Version = 0;    									// Версия. В нашем поиске не используется                   
            BitMask = 77;     									// Битовая маска. В нашем поиске не используется.                    
            DongleType = GrdDT.GSII64;				            // Ключ с поддержкой алгоритма GSII64 
            DongleModel = GrdFMM.ALL;					        // Все модели ключей. 
            DongleInterface = GrdFMI.ALL;		   		        // Интерфейс подключения любой (LPT или USB)

            // Устанавливаем параметры поиска ключей защиты.
            // Все последующие запуски функций GrdFind() и GrdLogin() перед следующий изменением параметров, функцией 
            // GrdSetFindMode() будут использовать установленные критерии поиска.
            // Если значения полей ключа и заданные не совпадут, то выдастся ошибка.
            RetCode = GrdApi.GrdSetFindMode(GrdHandle,
                                            RemoteMode,
                                            DongleFlags,
                                            ProgramNumber,
                                            DongleID,
                                            SerialNumber,
                                            Version,
                                            BitMask,
                                            DongleType,
                                            DongleModel,
                                            DongleInterface);
            ErrorHandling(GrdHandle, RetCode);

            /* Устанавливаем режим условной компиляции, данный код в итоговое приложение вставлен не будет */
            // Производим подключение к ключу (с параметрами поиска заданными GrdFindMode).
            // Все последующие вызовы функций Guardant API, до следующего вызова GrdCloseHandle()/GrdLogin()
            // будут использовать подключенный ключ
            RetCode = GrdApi.GrdLogin(GrdHandle, 0xFFFFFFFF, GrdLM.PerStation);
            ErrorHandling(GrdHandle, RetCode);

            /* ----------- Конец блока работы с Guardant API ------------------ */


            // Если файл конфигурации существует
            if (File.Exists("config.xml"))
            {
                try
                {
                    // Если файл настроек обнаружен пытаемся с ним работать через сериалайзер
                    Config = new config();
                    XmlSerializer serializer = new XmlSerializer(typeof(config));
                    TextReader reader = new StreamReader("config.xml");
                    Config = (config)serializer.Deserialize(reader);
                    reader.Close();

                    // Декодируем строку из BASE64 в byte
                    byte[] DBPathByte = DecodeFrom64(Config.DBPath);

                    /* ----------- Начало блока работы с Guardant API ------------------ */
                    // Дешифруем строку с использованием алгоритма GSII64
                    RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                                01,                         // Номер аппаратного алгоритма
                                DBPathByte,                 // Массив с путем к БД
                                (UInt32)GrdAM.OFB + (UInt32)GrdAM.Decode,   // Методы декодирования
                                InitVector);	            // Вектор инициализации
                    ErrorHandling(GrdHandle, RetCode);
                    /* ----------- Начало блока работы с Guardant API ------------------ */

                    // Получаем путь в string из массива байт
                    string DBPath = ByteToStr(DBPathByte);
                    
                    // из считанных настроек формируем строку соединения с БД
                    connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + DBPath;
                    

                }
                catch
                {
                    return;
                }
            }
            else
            {
                // Создаем и вызываем диалог выборов параметров подключения
                ConnectToDatabase СTDForm = new ConnectToDatabase();
                DialogResult res = СTDForm.ShowDialog();


                // Если строка пути к файлу БД пустая, то завершаем работу приложения
                if (СTDForm.TBFileName.Text == "") 
                {
                    Application.Exit();
                }

                /* ----------- Начало блока работы с Guardant API ------------------ */
                // Преобразуем Путь к БД из string в byte
                byte[] DBPathByte = StrToByte(СTDForm.TBFileName.Text);

                // Шифруем строку с использованием алгоритма GSII64
                RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                            01,                         // Номер аппаратного алгоритма
                            DBPathByte,                 // Массив с путем к БД
                            (UInt32)GrdAM.OFB + (UInt32)GrdAM.Encode,   // Методы кодирования
                            InitVector);	            // Вектор инициализации
                ErrorHandling(GrdHandle, RetCode);
                /* ----------- Начало блока работы с Guardant API ------------------ */

                // если файла настроек нет, то формируем парамерты по умолчанию
                // и записываем их в файл настроек для дальнейшего использования
                // запись через сериалайзер
                Config = new config();
                Config.DBPath = EncodeTo64(DBPathByte); // Преобразуем путь к БД в base64
                Config.DBAdmin = СTDForm.TBUserName.Text;
                Config.DBPassword = СTDForm.TBPassword.Text;
                XmlSerializer serializer = new XmlSerializer(typeof(config));
                TextWriter writer = new StreamWriter("config.xml");
                serializer.Serialize(writer, Config);
                writer.Close();
                connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + СTDForm.TBFileName.Text;

            }
            /* ----------- Начало блока работы с Guardant API ------------------ */


            // Проверяем на возможные ошибки
            ErrorHandling(GrdHandle, RetCode);
            /* ----------- Начало блока работы с Guardant API ------------------ */

            // Загрузка таблицы
            this.reloadTable();
            this.reloadTableShopping();
        }

        private void reloadTable()
        {
            // чистим таблицу
            this.dataGridView1.Rows.Clear();
            // созадем подключение и открываем его
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            // строка выбора всех полей из таблицы
            string command = "select * from [ClientsList]";
            // создаем класс запроса
            OleDbCommand cmd = new OleDbCommand(command, conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nRow;
            // пока таблица дает строки работает цикл
            while (myOleDbDataReader.Read())
            {
                // добавляем строку и заполняем колонки
                nRow = this.dataGridView1.Rows.Add();
                this.dataGridView1.Rows[nRow].Cells["ColumnID"].Value = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                this.dataGridView1.Rows[nRow].Cells["ColumnName"].Value = myOleDbDataReader["ClientName"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnAddress"].Value = myOleDbDataReader["ClientAddress"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnPhone"].Value = myOleDbDataReader["ClientTelNumber"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnComment"].Value = myOleDbDataReader["ClientComments"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnCity"].Value = myOleDbDataReader["ClientCity"].ToString();
            }
            // закрываем открытые подключения
            myOleDbDataReader.Close();
            conn.Close();
        }

        private void reloadTableShopping()
        {
            // чистим таблицу
            this.dataGridView2.Rows.Clear();
            // созадем подключение и открываем его
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            // строка выбора всех полей из таблицы
            string command = "select * from [Shopping]";
            // создаем класс запроса
            OleDbCommand cmd = new OleDbCommand(command, conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nRow;
            // пока таблица дает строки работает цикл
            while (myOleDbDataReader.Read())
            {
                // добавляем строку и заполняем колонки
                nRow = this.dataGridView2.Rows.Add();
                this.dataGridView2.Rows[nRow].Cells["ID"].Value = Convert.ToInt32(myOleDbDataReader["ID"].ToString());
                this.dataGridView2.Rows[nRow].Cells["ClientID2"].Value = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                DateTime PurchaseDate = Convert.ToDateTime( myOleDbDataReader["PurchaseDate"].ToString());
                this.dataGridView2.Rows[nRow].Cells["PurchaseDate"].Value = PurchaseDate.ToString("dd.MM.yyyy");
                this.dataGridView2.Rows[nRow].Cells["PurchaseName"].Value = myOleDbDataReader["PurchaseName"].ToString();
            }
            // закрываем открытые подключения
            myOleDbDataReader.Close();
            conn.Close();
        }
   
        //----------------------------------------------------------------------------------------------
        //  Функция обработки ошибок
        //  Показывает результат, закрывает хэндл, и завершает работу программы
        //  Вход:  Хэндл контейнера Guardant 
        //	Вход :  код ошибки
        //	Выход: код ошибки
        //----------------------------------------------------------------------------------------------
        private static GrdE ErrorHandling(Handle hGrd, GrdE nRet)
        {
            if (nRet != GrdE.OK)
            {   

                // Выводим ошибку последней функции
                MessageBox.Show(GrdApi.PrintResult((int)nRet), "Ошибка");

                // Отключаемся от ключа защиты
                nRet = GrdApi.GrdLogout(hGrd);
                if (hGrd.Address != 0)	// Проверяем существует ли хэндл контейнера Guardant 
                {
                    // Закрываем хэндл, выходим с сервера ключа,  освобождаем память
                    nRet = GrdApi.GrdCloseHandle(hGrd);
                }
                // Деинициализация копии Guardant API. Метод GrdCleanup() обязательно должен быть вызван перед закрытием программы
                nRet = GrdApi.GrdCleanup();

                // Terminate application
                Environment.Exit((int)nRet);
            }
            return nRet;
        }

        // Проверка наличия HelloWord в массиве
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            /* ----------- Начало блока работы с Guardant API ------------------ */

            if (GrdHandle.Address != 0) // Проверяем был ли создан хэндл
            {
                // Закрывает сеанс работы с ключом
                RetCode = GrdApi.GrdLogout(GrdHandle);
                ErrorHandling(GrdHandle, RetCode);

                // Закрытие хэндла контейнера Guardant. Выход с сервера ключа и освобождение памяти
                RetCode = GrdApi.GrdCloseHandle(GrdHandle);
                ErrorHandling(GrdHandle, RetCode);
            }

            // -----------------------------------------------------------------
            // Деинициализация копии GrdAPI. 
            // Функция GrdCleanup() должна быть вызвана последней перед закрытием программы
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdCleanup();
            ErrorHandling(GrdHandle, RetCode);
            /* ----------- Конец блока работы с Guardant API ------------------ */
        }
 
        // Byte в Base64
        static public string EncodeTo64(byte[] toEncodeAsBytes)
        {
            string returnValue
                  = System.Convert.ToBase64String(toEncodeAsBytes);
            return returnValue;
        }

        // Base64 в byte
        static public byte[] DecodeFrom64(string encodedData)
        {
            byte[] encodedDataAsBytes
                = System.Convert.FromBase64String(encodedData);
            return encodedDataAsBytes;
        }
        // Строка в Byte
        static public byte[] StrToByte(string EncodeStr) 
        {
            byte[] returnValue
                = System.Text.ASCIIEncoding.ASCII.GetBytes(EncodeStr);
            return returnValue;
        }
        // Byte в строку
        static public string ByteToStr(byte[] EncodeByte) 
        {
            string returnValue =
                System.Text.ASCIIEncoding.ASCII.GetString(EncodeByte);
            return returnValue;
        }

        // Byte в Unicode строку
        static public string ByteToUStr(byte[] EncodeByte)
        {
            string returnValue = System.Text.ASCIIEncoding.Unicode.GetString(EncodeByte);
            return returnValue;
        }


        // Unicode Строка в Byte
        static public byte[] UStrToByte(string EncodeStr)
        {
            byte[] returnValue = System.Text.ASCIIEncoding.Unicode.GetBytes(EncodeStr);
            return returnValue;
        }

        // Получение строки из Hex
        static public string ByteArrayToHexString(byte[] Bytes)  
        {  
            StringBuilder Result = new StringBuilder();  
            string HexAlphabet = "0123456789ABCDEF";  
            foreach (byte B in Bytes)  
            {
                Result.Append("0x");
                Result.Append(HexAlphabet[(int)(B >> 4)]);  
                Result.Append(HexAlphabet[(int)(B & 0xF)]);
                Result.Append(", ");
            }  
            return Result.ToString();  
        }
        // ------------- Шифрование таблицы клиентов ----------------------
        private void BtnCryptClientBase_Click(object sender, EventArgs e)
        {
            // Подключаемся к БД
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            OleDbCommand cmd = new OleDbCommand("select [ClientID], [ClientComments] from ClientsList", conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();

            // Проверяем есть ли данные в выборке
            if (myOleDbDataReader.HasRows)
            {
                // Перебираем все данные и шифруем их
                while (myOleDbDataReader.Read())
                {
                    ClientIDStr = myOleDbDataReader["ClientID"].ToString(); // Получаем ID клиента
                    ClientIDIntV = Convert.ToUInt32(ClientIDStr); // Преобразуем его в беззнаковое целое
                    ClientIDIntV = ClientIDIntV ^ 769390234;      // Модифицируем его для использования в качестве вектора инициализации
                    ClientComments = myOleDbDataReader["ClientComments"].ToString(); // Получаем комментарии
                    ClientCommentsB = UStrToByte(ClientComments); // Преобразуем в массив byte

                    // Шифруем строку комментариев на алгоритме GSII64 по методу OFB
                    RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                        04,                                 // Номер аппаратного алгоритма
                        ClientCommentsB,                    // Массив с комментарием 
                        (UInt32)GrdAM.OFB + (UInt32)GrdAM.Encode,   // Методы кодирования
                        ClientIDIntV);	                    // Вектор инициализации
                    ErrorHandling(GrdHandle, RetCode);
                    
                    ClientComments = EncodeTo64(ClientCommentsB); // Преобразуем в Base64

                    OleDbCommand cmdUpdate = new OleDbCommand("UPDATE ClientsList SET [ClientComments]='" + ClientComments+"' WHERE [ClientID]="+ClientIDStr, conn);
                    cmdUpdate.ExecuteReader();


                    
                }
            }
            conn.Close();                 // Закрываем соединение с БД
            reloadTable();
        }
        // ------------- Дешифрование таблицы клиентов ----------------------
        private void btnDecryptClientBase_Click(object sender, EventArgs e)
        {
            // Подключаемся к БД
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            OleDbCommand cmd = new OleDbCommand("select [ClientID], [ClientComments] from ClientsList", conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();

            // Проверяем есть ли данные в выборке
            if (myOleDbDataReader.HasRows)
            {
                // Перебираем всех клиентов данные и дешифруем их
                while (myOleDbDataReader.Read())
                {
                    ClientIDStr = myOleDbDataReader["ClientID"].ToString(); // Получаем ID клиента
                     ClientIDIntV = Convert.ToUInt32(ClientIDStr);
                    ClientIDIntV = ClientIDIntV ^ 769390234;
                    ClientComments = myOleDbDataReader["ClientComments"].ToString();
                    ClientCommentsB = DecodeFrom64(ClientComments); // Преобразуем в массив byte

                    // Дешифруем строку комментариев на алгоритме GSII64 по методу OFB
                    RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                        04,                                 // Номер аппаратного алгоритма
                        ClientCommentsB,                    // Массив с комментарием 
                        (UInt32)GrdAM.OFB + (UInt32)GrdAM.Decode,   // Дешифруем по методу OFB
                        ClientIDIntV);	                    // Вектор инициализации
                    ErrorHandling(GrdHandle, RetCode);

                    ClientComments = ByteToUStr(ClientCommentsB); // Преобразуем массив в строку Unicode

                    OleDbCommand cmdUpdate = new OleDbCommand("UPDATE ClientsList SET [ClientComments]='" + ClientComments + "' WHERE [ClientID]=" + ClientIDStr, conn);
                    cmdUpdate.ExecuteReader();



                }
            }
            conn.Close();                 // Закрываем соединение с БД
            reloadTable();
        }

        // ------------- Шифрование таблицы покупок  ----------------------
        private void BtnCryptShoppingBase_Click(object sender, EventArgs e)
        {
            InitVectorAES = StrToByte(InitVectorAES1.ToString("x08") + InitVectorAES2.ToString("x08"));
            
            // Подключаемся к БД
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            OleDbCommand cmd = new OleDbCommand("select [ClientID], [ClientName] from ClientsList", conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();

            // Проверяем есть ли данные в выборке
            if (myOleDbDataReader.HasRows)
            {

                // Перебираем всех клиентов и ищем у каждого покупки
                while (myOleDbDataReader.Read())
                {
                    ClientID = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                    ClientName = myOleDbDataReader["ClientName"].ToString();
                    OleDbCommand cmdShopping = new OleDbCommand("select [ID],[PurchaseName] from Shopping WHERE [ClientID]=" + ClientID.ToString(), conn);
                    // выполняем запрос получения всех покупок клиента
                    OleDbDataReader myOleDbDataReaderShopping = cmdShopping.ExecuteReader();
                    // Проверяем есть ли данные в выборке
                    if (myOleDbDataReaderShopping.HasRows)
                    {
                        // Создаем специальный ключ из ID и имени клиента
                        SecretKeyAESStr = ClientID.ToString("x04") + ClientName;
                        // Если длина специального ключа  меньше 32 циклично заполняем оставшуюся часть ID клиента
                        while (SecretKeyAESStr.Length < 33) { SecretKeyAESStr += ClientID.ToString(); }
                        SecretKeyAESStr = SecretKeyAESStr.Substring(0,32); // Обрезаем до 32 символов
                        // Преобразуем в массив byte[]
                        SecretKeyAES = StrToByte(SecretKeyAESStr);

                        // Шифруем специальный ключ шифрования AES на алгоритме GSII64 по методу ECB
                        RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                            04,                                 // Номер аппаратного алгоритма
                            SecretKeyAES,                    // Массив со специальным ключом 
                            (UInt32)GrdAM.ECB + (UInt32)GrdAM.Encode,   // Методы кодирования
                            0); // В данном методе вектор инициализации не используется
                        ErrorHandling(GrdHandle, RetCode);

                        // Перебираем все покупки у клиента
                        while (myOleDbDataReaderShopping.Read())
                        {
                            // Получаем данные из базы
                            string IDShopping = myOleDbDataReaderShopping["ID"].ToString();
                            string PurchaseName = myOleDbDataReaderShopping["PurchaseName"].ToString();
                            byte[] PurchaseNameB = UStrToByte(PurchaseName);

                            // Шифрование на программно-реализованном алгоритме AES256 
                            RetCode = GrdApi.GrdCrypt(GrdHandle,	// Хэндл контейнера Guardant
                                GrdSC.AES256,				    // Номер программно-реализованного алгоритма AES 
                                PurchaseNameB.Length,           // Длина данных для шифрования
                                PurchaseNameB,				    // Данные для шифрования
                                (UInt32)GrdAM.Encrypt +
                                (UInt32)GrdAM.CFB +
                                (UInt32)GrdSC.All, 			    // Шифровать в режиме CFB, единственный блок данных
                                InitVectorAES,			        // Вектор инициализации
                                SecretKeyAES,					// Секретный ключ шифрования/дешифрования
                                GrdAES_Context);			    // Буфер для преобразования
                            ErrorHandling(GrdHandle, RetCode); // Проверяем результат работы метода

                            // Преобразуем результат в Base64
                            PurchaseName = EncodeTo64(PurchaseNameB);

                            OleDbCommand cmdUpdate = new OleDbCommand("UPDATE Shopping SET [PurchaseName]='" + PurchaseName + "' WHERE [ID]=" + IDShopping, conn);
                            cmdUpdate.ExecuteReader();

                        }
                   
                    }    
  
                }
            }
            conn.Close();                 // Закрываем соединение с БД
            reloadTableShopping();

        }
        // ------------- Дешифрование таблицы покупок  ----------------------
        private void BtnDeCryptShoppingBase_Click(object sender, EventArgs e)
        {
            InitVectorAES = StrToByte(InitVectorAES1.ToString("x08") + InitVectorAES2.ToString("x08"));

            // Подключаемся к БД
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            OleDbCommand cmd = new OleDbCommand("select [ClientID], [ClientName] from ClientsList", conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();

            // Проверяем есть ли данные в выборке
            if (myOleDbDataReader.HasRows)
            {

                // Перебираем всех клиентов и ищем у каждого покупки
                while (myOleDbDataReader.Read())
                {
                    ClientID = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                    ClientName = myOleDbDataReader["ClientName"].ToString();
                    OleDbCommand cmdShopping = new OleDbCommand("select [ID],[PurchaseName] from Shopping WHERE [ClientID]=" + ClientID.ToString(), conn);
                    // выполняем запрос получения всех покупок клиента
                    OleDbDataReader myOleDbDataReaderShopping = cmdShopping.ExecuteReader();
                    // Проверяем есть ли данные в выборке
                    if (myOleDbDataReaderShopping.HasRows)
                    {
                        // Создаем вектор шифрования из ID и имени клиента
                        SecretKeyAESStr = ClientID.ToString("x04") + ClientName;
                        // Если длина вектора меньше 32 циклично заполняем оставшуюся часть ID клиента
                        while (SecretKeyAESStr.Length < 33) { SecretKeyAESStr += ClientID.ToString(); }
                        SecretKeyAESStr = SecretKeyAESStr.Substring(0, 32); // Обрезаем до 32 символов
                        // Преобразуем в массив byte[]
                        SecretKeyAES = StrToByte(SecretKeyAESStr);

                        // Шифруем вектор AES на алгоритме GSII64 по методу ECB
                        RetCode = GrdApi.GrdCrypt(GrdHandle,    // Хэндл контейнера Guardant
                            04,                                 // Номер аппаратного алгоритма
                            SecretKeyAES,                    // Массив с вектором инициализации
                            (UInt32)GrdAM.ECB + (UInt32)GrdAM.Encode,   // Методы кодирования
                            0); // В данном методе шифрования вектор инициализации не используется
                        ErrorHandling(GrdHandle, RetCode);

                        // Перебираем все покупки у клиента
                        while (myOleDbDataReaderShopping.Read())
                        {
                            // Получаем данные из базы
                            string IDShopping = myOleDbDataReaderShopping["ID"].ToString();
                            string PurchaseName = myOleDbDataReaderShopping["PurchaseName"].ToString();
                            // Преобразуем из Base64
                            byte[] PurchaseNameB = DecodeFrom64(PurchaseName);

                            // Дешифрование на программно-реализованном алгоритме AES256 
                            RetCode = GrdApi.GrdCrypt(GrdHandle,	// Хэндл контейнера Guardant
                                GrdSC.AES256,				    // Номер программно-реализованного алгоритма AES 
                                PurchaseNameB.Length,           // Длина данных для шифрования
                                PurchaseNameB,				    // Данные для дешифрования
                                (UInt32)GrdAM.Decrypt +
                                (UInt32)GrdAM.CFB +
                                (UInt32)GrdSC.All, 			    // Дешифровать в режиме CFB, единственный блок данных
                                InitVectorAES,			        // Вектор инициализации
                                SecretKeyAES,					// Секретный ключ шифрования/дешифрования
                                GrdAES_Context);			    // Буфер для преобразования
                            ErrorHandling(GrdHandle, RetCode);

                            // Получаем Unicode строку из массива байт
                            PurchaseName = ByteToUStr(PurchaseNameB);

                            OleDbCommand cmdUpdate = new OleDbCommand("UPDATE Shopping SET [PurchaseName]='" + PurchaseName + "' WHERE [ID]=" + IDShopping, conn);
                            cmdUpdate.ExecuteReader();

                        }

                    }

                }
            }
            conn.Close();                 // Закрываем соединение с БД
            reloadTableShopping();
        }








    }
}
