﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.Serialization;
using System.Data.OleDb;
using Guardant; // Используем классы Guardant

namespace Clients
{
    public partial class Form1 : Form
    {
        private config Config;
        private string connectionString;
        private bool lFilter;
        
        /* Начало. Блок переменных для работы с Guardant API */ 
        GrdE RetCode;                       // Код ошибки для всех функций Guardant API
        Handle GrdHandle = new Handle();    // Создает пустой хэндл для контейнера Guardant

        static uint CryptPU = 0x8566783U;         // Константы для шифрования кодов
        static uint CryptRD = 0x17d49c84U;        // доступа к ключу защиты


        // Шифруем демонстрационные коды доступа, вычитая из них случайные числа для 
        // обеспечения более высокой степени взломоучтойчивости
        // This way they are not shown in the application
        uint PublicCode = (uint)GrdDC.DEMONVK - CryptPU;    // Должно быть зашифровано        
        uint ReadCode = (uint)GrdDC.DEMORDO - CryptRD;    // Должно быть зашифровано
        
        // Переменные используются в функции GrdSetFindMode()
        GrdFMR RemoteMode;         // Флаги поиска локальных или сетевых ключей                  
        GrdFM DongleFlags;        // Флаги поиска ключа                  
        uint ProgramNumber = 0;  // Номер программы                      
        uint Version = 0;        // Версия программы                                 
        uint DongleID = 0;       // ID ключа (Номер ключа)
        byte[] TypeID = new byte[2];// Тип ключа
        uint SerialNumber;       // Сериный номер                           
        uint BitMask;            // Битовая маска                           
        GrdDT DongleType;         // Тип ключа                            
        GrdFMM DongleModel;        // Модель ключа                           
        GrdFMI DongleInterface;    // Тип интерфейса ключа

        FindInfo GrdFindInfo1 = new FindInfo();   // структура используемая в функции GrdFind()
        FindInfo GrdFindInfo2 = new FindInfo();   // структура используемая в функции GrdFind()


        // переменные для GrdWrite() и GrdRead()
        UInt32 TempDataID;              // Временные данные
        Byte[] TempData = new Byte[32]; // Временные данные для работы с функцией GrdRead
        Byte[] PIData = new Byte[32];   // Переменная для работы с защищенным ячейками

        /* Конец. Блок переменных для работы с Guardant API */

        int ClientID; // Хранится ID клиента из БД
        string ClientName; // Хранится имя клиента из БД


        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            if (File.Exists("config.xml"))
            {
                try
                {
                    // Если файл настроек обнаружен пытаемся с ним работать через сериалайзер
                    Config = new config();
                    XmlSerializer serializer = new XmlSerializer(typeof(config));
                    TextReader reader = new StreamReader("config.xml");
                    Config = (config)serializer.Deserialize(reader);
                    reader.Close();
                    // из считанных настроек формируем строку соединения с БД
                    connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + Config.DBPath;
                    

                }
                catch
                {
                    return;
                }
            }
            else
            {
                ConnectToDatabase СTDForm = new ConnectToDatabase();

                DialogResult res = СTDForm.ShowDialog();

                if (СTDForm.TBFileName.Text == "") 
                {
                    Application.Exit();
                }

                // если файла настроек нет, то формируем парамерты по умолчанию
                // и записываем их в файл настроек для дальнейшего использования
                // запись через сериалайзер
                Config = new config();
                Config.DBPath = СTDForm.TBFileName.Text;
                Config.DBAdmin = СTDForm.TBUserName.Text;
                XmlSerializer serializer = new XmlSerializer(typeof(config));
                TextWriter writer = new StreamWriter("config.xml");
                serializer.Serialize(writer, Config);
                writer.Close();
                connectionString = "provider=Microsoft.Jet.OLEDB.4.0;data source=" + Config.DBPath;
            }
            // Загрузка таблицы
            // перед загрузкой сбрасываем признак установленного фильтра
            lFilter = false;
            this.reloadTable();

            /* ----------- Начало блока работы с Guardant API ------------------ */
            
            // Инициализация копии GrdAPI. GrdStartup() должен быть вызван перед первым вызовом функций GrdAPI 
            RetCode = GrdApi.GrdStartup(GrdFMR.Local);	// Мы будем использовать только локальные ключи
            ErrorHandling(GrdHandle, RetCode);

            // -----------------------------------------------------------------
            // Создание контейнера Guardant и получение его хэндла
            // -----------------------------------------------------------------
            GrdHandle = GrdApi.GrdCreateHandle(GrdCHM.MultiThread);
            if (GrdHandle.Address == 0) // Если найдена ошибка
            {					
                ErrorHandling(new Handle(0), GrdE.MemoryAllocation);
            }

            // -----------------------------------------------------------------
            // Установка кодов доступа к ключу защиты Guardant
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdSetAccessCodes(GrdHandle,	// хэндл контейнера Guardant 
                PublicCode + CryptPU,   // Публичный код. Используется для поиска ключа. Должен быть установлен
                ReadCode + CryptRD,     // Приватный код для чтения; Данный код используется для чтения незащищенных ячеек памяти;
                0,	// Приватный код для записи. В большинстве случаев, он не нужен. Старайтесь не включать его в итоговые версии приложения.
                0);  // Мастер код. Используется для редактирования параметров ключа. Никогда не вставляйте этот код в публичные версии приложения!
           
            ErrorHandling(GrdHandle, RetCode); // Обработка ошибок


            // -----------------------------------------------------------------
            // Устанавливаем параметры поиска ключа
            // -----------------------------------------------------------------
            RemoteMode = GrdFMR.Local; 				            // Поиск только локальных ключей
            DongleFlags = GrdFM.NProg & GrdFM.Mask;	            // Устанавливаем флаг поиска по номеру программы и маске
            ProgramNumber = 5;     								// Устанавливаем номер программы. В нашем поиске не используется.                
            DongleID = 0;       		     					// ID ключа. Не используется
            SerialNumber = 0;     							    // Серийный номер. В нашем поиске не используется.                     
            Version = 0;    									// Версия. В нашем поиске не используется                   
            BitMask = 77;     									// Битовая маска. В нашем поиске не используется.                    
            DongleType = GrdDT.GSII64;				            // Ключ с поддержкой алгоритма GSII64 
            DongleModel = GrdFMM.ALL;					        // Все модели ключей. 
            DongleInterface = GrdFMI.ALL;		   		        // Интерфейс подключения любой (LPT или USB)

            // Устанавливаем параметры поиска ключей защиты.
            // Все последующие запуски функций GrdFind() и GrdLogin() перед следующий изменением параметров, функцией 
            // GrdSetFindMode() будут использовать установленные критерии поиска.
            // Если значения полей ключа и заданные не совпадут, то выдастся ошибка.
            RetCode = GrdApi.GrdSetFindMode(GrdHandle,
                                            RemoteMode,
                                            DongleFlags,
                                            ProgramNumber,
                                            DongleID,
                                            SerialNumber,
                                            Version,
                                            BitMask,
                                            DongleType,
                                            DongleModel,
                                            DongleInterface);
            ErrorHandling(GrdHandle, RetCode);

            /* Устанавливаем режим условной компиляции, данный код в итоговое приложение вставлен не будет */
            #if (DEBUG) 

            int   donglefindcount = 0;
            // -----------------------------------------------------------------
            // Поиск всех ключей и вывод их ID
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdFind(GrdHandle, GrdF.First, out DongleID, out GrdFindInfo1);
            while (RetCode == GrdE.OK)
            {
                // Увеличиваем счетчик ключей на 1
                donglefindcount++;
                // Выводим ID всех ключей
                MessageBox.Show("ID " + Convert.ToString(donglefindcount) + "-го ключа: " + Convert.ToString(DongleID), "Поиск ключей");
                // Поиск следующего ключа
                RetCode = GrdApi.GrdFind(GrdHandle, GrdF.Next, out DongleID, out GrdFindInfo2);
            }
            if (RetCode == GrdE.AllDonglesFound)						// Поиск завершен?
                MessageBox.Show("Поиск ключей завершен");
            else
                ErrorHandling(GrdHandle, RetCode);

            if (donglefindcount == 0)
            { // Если ни один ключ не найден, выводим ошибку и закрываем программу
                MessageBox.Show("Ни одного ключа защиты Guardant не найдено", "Поиск ключей");
                Application.Exit(); // Завершаем программу
            }
            else
            {   // Выводим подробную информацию о первом найденном ключе из структуры GrdFindInfo1
                MessageBox.Show(
                    "Подробная информация о первом найденном ключе: " + (char)10+(char)13+
                    "ID ключа = " + Convert.ToString(GrdFindInfo1.dwID) + (char)10+(char)13+
                    "Публичный код = 0x"+ GrdFindInfo1.dwPublicCode.ToString("X")+ (char)10+(char)13 +
                    "Версия прошивки = " + Convert.ToString(GrdFindInfo1.byHrwVersion)  + (char)10+(char)13+
                    "Ресурс сетевого ключа (макс) = " + Convert.ToString(GrdFindInfo1.byMaxNetRes) + (char)10+(char)13+
                    "Тип ключа = " + GrdFindInfo1.wType.ToString("X") + (char)10 + (char)13 +
                    "Номер программы = " + Convert.ToString(GrdFindInfo1.byNProg) + (char)10+(char)13+
                    "Версия программы = " + Convert.ToString(GrdFindInfo1.byVer) + (char)10+(char)13+
                    "Серийный номер = " + Convert.ToString(GrdFindInfo1.wSN) + (char)10+(char)13+
                    "Битовая маска  = " + Convert.ToString( GrdFindInfo1.wMask)+ (char)10+(char)13+
                    "Счетчик  = " + Convert.ToString( GrdFindInfo1.wGP)+ (char)10+(char)13+
                    "Реальный сетевой ресурс =  " + Convert.ToString( GrdFindInfo1.wRealNetRes)+ (char)10+(char)13+
                    "Индекс = " + Convert.ToString( GrdFindInfo1.dwIndex)
                , "Информация о ключе");            
            }

            #endif // конец блока условной компиляции

            // Производим подключение к ключу (с параметрами поиска заданными GrdFindMode).
            // Все последующие вызовы функций Guardant API, до следующего вызова GrdCloseHandle()/GrdLogin()
            // будут использовать подключенный ключ
            RetCode = GrdApi.GrdLogin(GrdHandle, 0xffffffff, GrdLM.PerStation);
            ErrorHandling(GrdHandle, RetCode);

            /* ----------- Конец блока работы с Guardant API ------------------ */

        }

        private void reloadTable()
        {
            // чистим таблицу
            this.dataGridView1.Rows.Clear();
            // созадем подключение и открываем его
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            // строка выбора всех полей из таблицы
            string command = "select * from [ClientsList]";
            if (lFilter)
            {
                // если взведен признак фильтра, то формируем ту часть запроса
                // которая отвечает за условие отбора
                // при этом используется команда LIKE, которая позволяет делать сравнение на вхождение по маске
                string cName, cPhone, cAddress;
                bool lNoWhere = false;
                cName = this.textBox3.Text;
                cPhone = this.textBox5.Text;
                cAddress = this.textBox4.Text;
                if (cName.Length > 0)
                {
                    lNoWhere = true;
                    // на всякий случай заменяем одинарные кавычки на двойные, а знак процента на двойной процент - это необходимо чтобы не завалить БД
                    // ЗЫ: это один из способов ломануть сайт, построенный на субд, если программист забыл это предусмотреть
                    command += " where [ClientName] like '%" + cName.Replace("'", "\"").Replace("%", "%%") + "%'";
                }

                if (cPhone.Length > 0)
                {

                    if (!lNoWhere)
                    {
                        command += " where ";
                        lNoWhere = true;
                    }
                    else
                        command += " and ";

                    command += "[ClientTelNumber] like '%" + cPhone.Replace("'", "\"").Replace("%", "%%") + "%'";
                }

                if (cAddress.Length > 0)
                {
                    if (!lNoWhere)
                    {
                        command += " where ";
                        lNoWhere = true;
                    }
                    else
                        command += " and ";

                    command += "[ClientAddress] like '%" + cAddress.Replace("'", "\"").Replace("%", "%%") + "%'";
                }
            }
            // создаем класс запроса
            OleDbCommand cmd = new OleDbCommand(command, conn);
            // выполняем запрос с получение объекта результирующей таблицы
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nRow;
            // пока таблица дает строки работает цикл
            while (myOleDbDataReader.Read())
            {
                // добавляем строку и заполняем колонки
                nRow = this.dataGridView1.Rows.Add();
                this.dataGridView1.Rows[nRow].Cells["ColumnID"].Value = Convert.ToInt32(myOleDbDataReader["ClientID"].ToString());
                this.dataGridView1.Rows[nRow].Cells["ColumnName"].Value = myOleDbDataReader["ClientName"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnAddress"].Value = myOleDbDataReader["ClientAddress"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnPhone"].Value = myOleDbDataReader["ClientTelNumber"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnComment"].Value = myOleDbDataReader["ClientComments"].ToString();
                this.dataGridView1.Rows[nRow].Cells["ColumnCity"].Value = myOleDbDataReader["ClientCity"].ToString();
            }
            // закрываем открытые подключения
            myOleDbDataReader.Close();
            conn.Close();
            // включаем или выключаем кнопки на панели инструментов
            bindingNavigatorAddNewItem.Enabled = true;
            if (dataGridView1.Rows.Count > 0)
            {
                bindingNavigatorMoveFirstItem.Enabled = true;
                bindingNavigatorMovePreviousItem.Enabled = true;
                bindingNavigatorMoveNextItem.Enabled = true;
                bindingNavigatorMoveLastItem.Enabled = true;
                bindingNavigatorDeleteItem.Enabled = true;
                dataGridView1.Rows[0].Selected = true;
            }
            else
            {
                bindingNavigatorMoveFirstItem.Enabled = false;
                bindingNavigatorMovePreviousItem.Enabled = false;
                bindingNavigatorMoveNextItem.Enabled = false;
                bindingNavigatorMoveLastItem.Enabled = false;
                bindingNavigatorDeleteItem.Enabled = false;
            }
        }


        private void bindingNavigatorMoveFirstItem_Click(object sender, EventArgs e)
        {
            // переход на первую строку таблицы
            dataGridView1.CurrentCell = dataGridView1.Rows[0].Cells[dataGridView1.CurrentCell.ColumnIndex];
        }

        private void bindingNavigatorMovePreviousItem_Click(object sender, EventArgs e)
        {
            // переход на предыдущую строку таблицы
            if (dataGridView1.CurrentRow.Index > 0)
            {
                dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.CurrentRow.Index - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
            }
        }

        private void bindingNavigatorMoveNextItem_Click(object sender, EventArgs e)
        {
            // переход на следующую строку таблицы
            if (dataGridView1.CurrentRow.Index < dataGridView1.Rows.Count-1)
            {
                dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.CurrentRow.Index + 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
            }
        }

        private void bindingNavigatorMoveLastItem_Click(object sender, EventArgs e)
        {
            // переход на последнюю строку таблицы
            dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
        }

        private void bindingNavigatorAddNewItem_Click(object sender, EventArgs e)
        {
            // ----------------- Добавление новой записи ------------------------
            // Создаем и открываем форму добавления новой записи
            InsertToDB InsertForm = new InsertToDB();
            DialogResult res = InsertForm.ShowDialog();
           
            // Если нажата кнопка "Добавить"
            if (InsertForm.PushBtnAdd)
            {
                // открываем подключение
                AESBase ClientCity = new AESBase();
                OleDbConnection conn = new OleDbConnection(this.connectionString);
                conn.Open();
                // получаем максимальную код идентификатора
                string command = "INSERT INTO [ClientsList] ([ClientName], [ClientAddress], [ClientTelNumber], [ClientCity], [ClientComments]) "+
                    "VALUES ('" + InsertForm.TBClientName.Text + "', '" + InsertForm.TBClientAddress.Text + "' ,'" + InsertForm.TBClientTel_Prefix.Text + " (" + InsertForm.CBClientTel_Code.Text + ") " + InsertForm.TBClientTel_fix.Text + "', '" +
                    ClientCity.CityBase[InsertForm.CBClientTel_Code.Text] + "' ,'" + InsertForm.TBClientComments.Text + "')";
                OleDbCommand cmd = new OleDbCommand(command, conn);
                OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
                conn.Close();
                this.reloadTable();
            }

/*
            // получаем максимальную код идентификатора
            string command = "select Max(ClientID) from [ClientsList]";
            OleDbCommand cmd = new OleDbCommand(command, conn);
            OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
            int nID;
            // если таблица не пустая, то берем максимум и наращиваем на один
            if (myOleDbDataReader.Read())
                nID = int.Parse(myOleDbDataReader[0].ToString())+1;
            // если таблица пустая, то первая запись
            else
                nID = 1;
            // закрываем табличку
            myOleDbDataReader.Close();
            // делаем новую запись с заполнением идентификатора
            cmd.CommandText = "INSERT INTO [ClientsList] ([ClientID]) VALUES (" + nID.ToString() + ")";
            if (cmd.ExecuteNonQuery() > 0)
            {
                // если все создалось, то делаем реинициализацию таблицы и заходим на последнюю строку таблицы (это вновь созданная строка)
                conn.Close();
                lFilter = false;
                button4.Enabled = false;
                this.reloadTable();
                this.dataGridView1.CurrentCell = dataGridView1.Rows[dataGridView1.Rows.Count - 1].Cells[dataGridView1.CurrentCell.ColumnIndex];
                // здесь выходим из метода, т.к. если этого не сделать - следующая строка закрытия выпадет
                return;
            }
            conn.Close();
 */ 
        }

        private void bindingNavigatorDeleteItem_Click(object sender, EventArgs e)
        {
            // удаление записи
            OleDbConnection conn = new OleDbConnection(this.connectionString);
            conn.Open();
            string idc = this.dataGridView1.CurrentRow.Cells[0].Value.ToString();
            string command = "delete from [ClientsList] where [ClientID] = "+idc;
            OleDbCommand cmd = new OleDbCommand(command, conn);
            if (cmd.ExecuteNonQuery() > 0)
            {
                // если команда отработала, то удаляем из таблицы текущую запись
                dataGridView1.Rows.Remove(dataGridView1.CurrentRow);
            }
            conn.Close();
            // если удалена последняя запись, то нужно выключить кнопки на панели инструментов
            if (dataGridView1.Rows.Count == 0)
            {
                bindingNavigatorMoveFirstItem.Enabled = false;
                bindingNavigatorMovePreviousItem.Enabled = false;
                bindingNavigatorMoveNextItem.Enabled = false;
                bindingNavigatorMoveLastItem.Enabled = false;
                bindingNavigatorDeleteItem.Enabled = false;
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            // выключаем фильтр
            lFilter = false;
            this.button4.Enabled = false;
            this.button3.Enabled = true;
            this.reloadTable();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            // включаем фильтр
            lFilter = true;
            this.button4.Enabled = true;
            this.button3.Enabled = false;
            this.reloadTable();
        }

        private void dataGridView1_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // событие завершения редактирования поля в таблице
            // получаем код клиента
            string ID = this.dataGridView1.Rows[e.RowIndex].Cells[0].Value.ToString();
            string namefld = "";
            // получаем новый текст, введенный пользователем
            string newdata = this.dataGridView1.Rows[e.RowIndex].Cells[e.ColumnIndex].Value.ToString();
            // определяем какое поле менялось
            switch (e.ColumnIndex)
            {
                case 1:
                    namefld = "ClientName";
                    break;
                case 2:
                    namefld = "ClientAddress";
                    break;
                case 3:
                    namefld = "ClientTelNumber";
                    break;
                case 4:
                    namefld = "ClientComments";
                    break;
            }
            // если не смогли определить, то выходим
            if (namefld.Length == 0)
                return;
            OleDbConnection conn = new OleDbConnection(connectionString);
            conn.Open();
            // формируем запрос обновления данных в таблице БД
            OleDbCommand cmd = new OleDbCommand("UPDATE ClientsList SET ["+namefld+"] = '" + newdata + "' where [ClientId] = " + ID, conn);
            cmd.ExecuteNonQuery();
            conn.Close();
        }
        /* ------------------- Функция поиска по имени ---------------------------- */
        private void buttonFindName_Click(object sender, EventArgs e)
        { 
            
            string name = this.TBFindName.Text;
            if (name.Length > 0) // Если имя введено
            {


                #if (DEBUG) // Будем включать данный код, только в тестовую версию приложения
                
                /*  ----------- Начало блока Guardant API -------- */

                // -----------------------------------------------------------------
                // Пробуем считать обычной функцией чтения из памяти защищенной ячейки
                RetCode = GrdApi.GrdRead(GrdHandle,         // Хэндл контейнера
                                            024,            // Адрес памяти в ключе для начала считывания
                                            032,            // Размер блока данных для считывания
                                            out TempData);  // Переменная, куда будет помещен считанный блок
                ErrorHandling(GrdHandle, RetCode);

                // -----------------------------------------------------------------
                //Переключаемся на адресацию памяти SAM
                RetCode = GrdApi.GrdSetWorkMode(GrdHandle, GrdWM.SAM, GrdWMFM.DriverAuto);
                ErrorHandling(GrdHandle, RetCode);

                // -----------------------------------------------------------------
                // Чтение поля ID ключа              
                RetCode = GrdApi.GrdRead(GrdHandle, GrdSAM.dwID, out TempDataID);
                ErrorHandling(GrdHandle, RetCode);

                // Выводим номер ключа
                MessageBox.Show("Номер ключа: " + TempDataID.ToString(), "Тест функции GrdRead");

                /*  ----------- Конец блока Guardant API -------- */
                #endif



                OleDbConnection conn = new OleDbConnection(connectionString);
                conn.Open();
                // делаем запрос на получение идентификатора по первому совпадению текста в подстроке
                OleDbCommand cmd = new OleDbCommand("select [ClientID] from ClientsList where [ClientName] like '%" + name + "%'", conn);
                OleDbDataReader myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    // если нашли, то берем идентификатор 
                    string id = myOleDbDataReader[0].ToString();
                    // и перебирая все записи находим нужную
                    foreach(DataGridViewRow row in dataGridView1.Rows)
                    {
                        if (row.Cells[0].Value.ToString() == id)
                        {
                            // если нашли, то делаем ее текущей и выходим из цикла (зачем лишние движения?)
                            dataGridView1.CurrentCell = row.Cells[0];
                            break;
                        }
                    }
                } 
                else 
                {
                    MessageBox.Show("Запись по данному запросу не найдена", "Ошибка");
                }

                myOleDbDataReader.Close();
                conn.Close();
            }
        } // конец функции поиска по имени

        
        // Функция поиск по телефону (описание см. выше)
        private void buttonFindTel_Click(object sender, EventArgs e)
        {
            // Создание генератора случайных чисел
            Random rnd = new Random();
            // Создание подключения к БД, инициализация переменных
            OleDbConnection conn = new OleDbConnection(connectionString);
            conn.Open();
            OleDbCommand cmd = new OleDbCommand();
            OleDbDataReader myOleDbDataReader = null;

            string name = this.TBFindTel.Text;
            if (name.Length > 0)
            {

/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      Внимание! Данный пример является демонстрационным. Не стоит необдуманно использовать все предлагаемые методы.
      При разработке защиты обязательно учитывайте возможные сбои в работе реального приложения,
      такие как сбой БД, удаление части записей вручную и другие. Помните, что защита должна быть
      не только надежной, но и гибкой, устойчивой к любым сбоям самого приложения и его данных.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/
         
/*  ----------- Начало блока Guardant API -------- */

            // В начале данного блока необходимо изменить маску ключа.
            // Запишите в ключ маску из файла "lesson1.nsd", делается
            // это с помощью программы GrdUtil.exe
            // Более подробно как это сделать можно прочитать в части 3 первого урока                

                // -----------------------------------------------------------------
                // Чтение данных из защищенной ячейки ключа защиты
                RetCode = GrdApi.GrdPI_Read(
                    GrdHandle,							// Хэндл контейнера Guardant
                    0,			                        // Номер защищенной ячейки 
                    0,									// Смещение с которого будет производиться чтение в защищенной ячейке
                    32,									// Количество байтов для чтения
                    out PIData,							// Указатель на буфер считанных данных
                    1469608341);			            // Пароль на чтение для защищенной ячейки
                
                // Если ячейка до сих пор не активирована. Первый запуск.
                if (RetCode == GrdE.InactiveItem)
                { // начало блока. ячейка не активирована

                    // -----------------------------------------------------------------
                    // Активация защищенной ячейки или алгоритма
                    RetCode = GrdApi.GrdPI_Activate(
                        GrdHandle,			 // Хэндл контейнера Guardant
                        0,				     // Номер защищенной ячейки или алгоритма
                        408956937);	         // Пароль для активации ячейки или алгоритма
                    ErrorHandling(GrdHandle, RetCode);

                    // -----------------------------------------------------------------
                    // Если активация проша успешно, производим чтение ячейки еще раз
                    RetCode = GrdApi.GrdPI_Read(
                        GrdHandle,							// Хэндл контейнера Guardant
                        0,			                        // Номер защищенной ячейки 
                        0,									// Смещение с которого будет производиться чтение в защищенной ячейке
                        32,									// Количество байтов для чтения
                        out PIData,							// Указатель на буфер считанных данных
                        1469608341);			            // Пароль на чтение для защищенной ячейки

                    ErrorHandling(GrdHandle, RetCode); // Проверяем корректность завершения функции чтения   

                    // Проверяем считанную ячейку. Есть ли там нет умолчального HelloWord!, выдаем ошибку
                    if (CheckHelloWord() == 0) {
                        MessageBox.Show("Обнаружено нарушение защиты", "Ошибка ключа защиты");
                        if (myOleDbDataReader != null) myOleDbDataReader.Close();
                        conn.Close();
                        Application.Exit();
                    };

                }   // конец блока. ячейка не активирована
                else
                { // начало блока. ячейка уже активирована
                    
                    ErrorHandling(GrdHandle, RetCode); // Проверяем корректность завершения функции чтения   

                    // Проверяем считанную ячейку. Есть ли она уже активирована, а там умолчальное HelloWord!, выдаем ошибку
                    if (CheckHelloWord() == 1) {
                        MessageBox.Show("Обнаружено нарушение защиты", "Ошибка ключа защиты");
                        if (myOleDbDataReader != null) myOleDbDataReader.Close();
                        conn.Close();
                        Application.Exit();
                    };
                    // Получаем номер клиента из защищенной ячейки
                    ClientID = PIData[3]+ PIData[2]*256 + PIData[1]*65536 + PIData[0]*16777216;

                    // Получаем имя клиента из защищенной ячейки
                    ClientName = "";
                    for (int i = 4; i < 33; i++)
                    {
                        if (PIData[i] == 0) break;
                        ClientName += (char)PIData[i];
                    }

                    // Проверяем существует ли клиент с данными полученными из защищенной ячейки в нашей Базе данных

                    cmd = new OleDbCommand("select TOP 1 [ClientID] from ClientsList where ([ClientID]=" + ClientID.ToString() +" AND [ClientName]='"+ClientName+"')", conn);
                    myOleDbDataReader = cmd.ExecuteReader();
                    bool ReadBool = myOleDbDataReader.Read(); // Записываем найдена ли ячейка

                    if (ReadBool) // Проверяем найдена ли ячейка?
                    { // Если найдена, то закрываем соединение с БД
                        //string ClientID = myOleDbDataReader[0].ToString();
                    }
                    else 
                    { // Если не найдена, выводим сообщение и закрываем программу
                        MessageBox.Show("Обнаружено нарушение защиты", "Ошибка ключа защиты");
                        if (myOleDbDataReader != null) myOleDbDataReader.Close();
                        conn.Close();
                        Application.Exit();                        
                    }

                } // конец блока. ячейка уже активирована

                /* ----------- Получаем запись из случайной ячейки БД и заносим ее в защищенную ячейку ----------------- */
                // Ищем самый максимальный ID
                cmd = new OleDbCommand("SELECT TOP 1 [ClientID] FROM ClientsList ORDER BY ClientID DESC", conn);
                myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    ClientID = Convert.ToInt32(myOleDbDataReader[0].ToString());
                }

                int randNum = rnd.Next(ClientID) + 1; // Делаем случайное число от максимального ID 

                // Выбираем случайную запись
                cmd = new OleDbCommand("SELECT TOP 1 [ClientID], [ClientName]  FROM ClientsList  WHERE [ClientID]>" + randNum.ToString(), conn);
                myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    ClientID = Convert.ToInt32(myOleDbDataReader[0].ToString());
                    ClientName = myOleDbDataReader[1].ToString();
                }
                        
               PIData[3] = Convert.ToByte(ClientID % 256); ClientID = ClientID / 256;
               PIData[2] = Convert.ToByte(ClientID % 256); ClientID = ClientID / 256;
               PIData[1] = Convert.ToByte(ClientID % 256); ClientID = ClientID / 256;
               PIData[0] = Convert.ToByte(ClientID);

                // Переносим строку в массив
               for (int i = 4; i < 32; i++)
               {
                   if (ClientName.Length>i-4) 
                   {
                       PIData[i] = Convert.ToByte(ClientName[i-4]);
                   } else {
                       PIData[i] = 0;
                   }
               }

               // -----------------------------------------------------------------
               // Запись данных в защищенную ячейку. 
               RetCode = GrdApi.GrdPI_Update(
                   GrdHandle,								// Хэндл контейнера 
                   0,				                        // Номер защищенной ячейки или алгоритма 
                   0,										// Смещение защищенной ячей
                   32,									    // Количество байтов для записи
                   PIData,							    // Указатель на буфер с данными
                   843783688,		                        //Пароль на запись, если нет то 0
                   GrdUM.MOV);						        // Метод обновления (MOV или XOR)
               ErrorHandling(GrdHandle, RetCode);
                       

/* ----------- Конец блока работы с Guardant API ------------------ */                

                // Поиск телефона по БД и переход к данной записи
                cmd = new OleDbCommand("select [ClientID] from ClientsList where [ClientTelNumber] like '%" + name + "%'", conn);
                myOleDbDataReader = cmd.ExecuteReader();
                if (myOleDbDataReader.Read())
                {
                    string id = myOleDbDataReader[0].ToString();
                    foreach (DataGridViewRow row in dataGridView1.Rows)
                    {
                        if (row.Cells[0].Value.ToString() == id)
                        {
                            dataGridView1.CurrentCell = row.Cells[0];
                            break;
                        }
                    }
                }
                else 
                {
                    MessageBox.Show("Запись по данному запросу не найдена", "Ошибка");
                }
            }

            // Завершаем работу с базой данных
            if (myOleDbDataReader != null ) myOleDbDataReader.Close();
            conn.Close();
        } // конец функции поиска по номеру телефона

        //----------------------------------------------------------------------------------------------
        //  Функция обработки ошибок
        //  Показывает результат, закрывает хэндл, и завершает работу программы
        //  Вход:  Хэндл контейнера Guardant 
        //	Вход :  код ошибки
        //	Выход: код ошибки
        //----------------------------------------------------------------------------------------------
        private static GrdE ErrorHandling(Handle hGrd, GrdE nRet)
        {
            if (nRet != GrdE.OK)
            {
                // Выводим ошибку последней функции
                MessageBox.Show(GrdApi.PrintResult((int)nRet), "Ошибка");

                if (hGrd.Address != 0)	// Проверяем существует ли хэндл контейнера Guardant 
                {
                    // Закрываем хэндл, выходим с сервера ключа,  освобождаем память
                    nRet = GrdApi.GrdCloseHandle(hGrd);
                }

                // Деинициализация копии Guardant API. Функция GrdCleanup() должна быть вызвана перед закрытием программы
                nRet = GrdApi.GrdCleanup();

                // Terminate application
                Environment.Exit((int)nRet);
            }
            return nRet;
        }

        // Проверка наличия HelloWord в массиве
        private int CheckHelloWord () 
        {
          // Проверяем считанную ячейку. Есть ли там умолчальное HelloWord!
          int i = 0; // Переменная цикла
                    string HelloWord = "HelloWord!";
                    while (i < 10)
                    { // длина "HelloWord!"
                        if (PIData[i] != HelloWord[i]) { break; } i++;
                    }

                    // Если HelloWord! отсутствует
                    if (i == 10) { return 1; } else { return 0; }
        }
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            /* ----------- Начало блока работы с Guardant API ------------------ */

            if (GrdHandle.Address != 0) // Проверяем был ли создан хэндл
            {
                // Закрывает сеанс работы с ключом
                RetCode = GrdApi.GrdLogout(GrdHandle);
                ErrorHandling(GrdHandle, RetCode);

                // Закрытие хэндла контейнера Guardant. Выход с сервера ключа и освобождение памяти
                RetCode = GrdApi.GrdCloseHandle(GrdHandle);
                ErrorHandling(GrdHandle, RetCode);
            }

            // -----------------------------------------------------------------
            // Деинициализация копии GrdAPI. 
            // Функция GrdCleanup() должна быть вызвана последней перед закрытием программы
            // -----------------------------------------------------------------
            RetCode = GrdApi.GrdCleanup();
            ErrorHandling(GrdHandle, RetCode);
            /* ----------- Конец блока работы с Guardant API ------------------ */
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void dataGridView1_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }


    }
}
